@@ -14,6 +14,7 @@ import time
 from selenium.webdriver.common.by import By
 from selenium.webdriver.support.ui import WebDriverWait
 from selenium.webdriver.support import expected_conditions as EC
+from selenium.common.exceptions import TimeoutException
 from pywinauto import Application, findwindows
 import pyautogui
 from msb_rpa.web import get_driver
@@ -21,6 +22,8 @@ from msb_rpa.web import get_driver_chrome_incognito
 from msb_rpa.web import get_driver_autodownload
 from msb_rpa.requests import get_driver_selenium_wire
 from msb_rpa.web import close_website_panes
+from msb_rpa.generelt import generate_password
+from msb_rpa.generelt import basic_email
 
 # Constants
 WINDOWS_USERNAME = os.getenv('USERNAME')
@@ -77,13 +80,16 @@ def fasit_login(username: str, password: str, timers=None, drivertype: str = 'ch
     print(logged_in_tekst)
 
     if drivertype == 'chrome_wire':
-        time.sleep(5)
+        time.sleep(8)
         # Iterate through captured requests to find the one with the Bearer token
         for request in driver.requests:
             if request.response and 'Authorization' in request.headers:
-                bearer_token = request.headers['Authorization'].split()[1]  # Extract Bearer token
-                close_website_panes()
-                return bearer_token
+                parts = request.headers['Authorization'].split()
+                # Check if the header has the correct format and starts with 'Bearer'
+                if len(parts) == 2 and parts[0] == 'Bearer':
+                    bearer_token = parts[1]  # Extract Bearer token
+                    close_website_panes()
+                    return bearer_token
 
         # If the loop completes without finding a bearer token, raise an exception
         raise Exception("Bearer token not found in captured requests.")
@@ -128,13 +134,18 @@ def bogv_login(username, password):
 
 
 # Opus Login
-def opus_login(username: str, password: str):
+def opus_login(username: str, password: str, credential_name: str):
     """
     Logger ind i desktop udgaven af OPUS
 
     Parameters:
         username (str): The username for login.
         password (str): The password for login.
+        credential_name (str): Used in case there is a need to update the password.
+
+    Returns:
+        None: If the login was successful.
+        New Password (str): If there was a need for a password change.
     """
     driver = get_driver()
 
@@ -153,20 +164,48 @@ def opus_login(username: str, password: str):
     element = wait.until(EC.element_to_be_clickable((By.ID, "buttonLogon")))
     element.click()
 
-    xpath_expression = '//div[@displayname="Mine Genveje" and starts-with(@id, "tabIndex")]'
-    element = wait.until(EC.element_to_be_clickable((By.XPATH, xpath_expression)))
-    element.click()
+    try:
+        wait.until(EC.element_to_be_clickable((By.ID, "topLevelNavigationTR")))
+    except TimeoutException:
+        new_pw = generate_password()
 
-    time.sleep(5)
-    files = os.listdir(DOWNLOAD_DIRECTORY)
-    for file in files:
-        if file.endswith("tx.sap"):
-            os.system(f"start {DOWNLOAD_DIRECTORY}\\{file}")
-            break
+        old_pw = wait.until(EC.visibility_of_element_located((By.XPATH, '//input[@id="inputUsername"]')))
+        old_pw.send_keys(password)
 
-    time.sleep(10)
-    _opus_aabn()
-    close_website_panes()
+        new_pw_first = wait.until(EC.visibility_of_element_located((By.XPATH, '//input[@name="j_sap_password"]')))
+        new_pw_first.send_keys(new_pw)
+
+        new_pw_second = wait.until(EC.visibility_of_element_located((By.XPATH, '//input[@name="j_sap_again"]')))
+        new_pw_second.send_keys(new_pw)
+
+        skift_button = wait.until(EC.visibility_of_element_located((By.XPATH, '//input[@id="changeButton"]')))
+        skift_button.click()
+
+        basic_email(["lfrmi@aarhus.dk", "du@msb.aarhus.dk"], f"Password ændret på {credential_name}",
+            f"""<html>
+            <body>
+            <p>Password på {credential_name} ændret fra {password} til {new_pw}</p>
+            <p></p>
+            <p>Husk at opdatere {credential_name} på OpenOrchestrator på Udvikling + Onenote</p>
+            </body>
+            </html>""")
+
+        xpath_expression = '//div[@displayname="Mine Genveje" and starts-with(@id, "tabIndex")]'
+        element = wait.until(EC.element_to_be_clickable((By.XPATH, xpath_expression)))
+        element.click()
+
+        time.sleep(5)
+        files = os.listdir(DOWNLOAD_DIRECTORY)
+        for file in files:
+            if file.endswith("tx.sap"):
+                os.system(f"start {DOWNLOAD_DIRECTORY}\\{file}")
+                break
+
+        time.sleep(10)
+        _opus_aabn()
+        close_website_panes()
+
+        return new_pw
 
 
 def _opus_aabn():
@@ -190,13 +229,18 @@ def _opus_aabn():
         print("vindue ikke fundet")
 
 
-def opus_online_login(username, password):
+def opus_online_login(username: str, password: str, credential_name: str):
     """
     Logger ind i online udgaven af OPUS
 
     Parameters:
         username (str): The username for login.
         password (str): The password for login.
+        credential_name (str): Used in case there is a need to update the password.
+
+    Returns:
+        None: If the login was successful.
+        New Password (str): If there was a need for a password change.
     """
     driver = get_driver()
     wait = WebDriverWait(driver, 10)
@@ -214,7 +258,33 @@ def opus_online_login(username, password):
     element = wait.until(EC.element_to_be_clickable((By.ID, "buttonLogon")))
     element.click()
 
-    wait.until(EC.element_to_be_clickable((By.ID, "topLevelNavigationTR")))
+    try:
+        wait.until(EC.element_to_be_clickable((By.ID, "topLevelNavigationTR")))
+    except TimeoutException:
+        new_pw = generate_password()
+
+        old_pw = wait.until(EC.visibility_of_element_located((By.XPATH, '//input[@id="inputUsername"]')))
+        old_pw.send_keys(password)
+
+        new_pw_first = wait.until(EC.visibility_of_element_located((By.XPATH, '//input[@name="j_sap_password"]')))
+        new_pw_first.send_keys(new_pw)
+
+        new_pw_second = wait.until(EC.visibility_of_element_located((By.XPATH, '//input[@name="j_sap_again"]')))
+        new_pw_second.send_keys(new_pw)
+
+        skift_button = wait.until(EC.visibility_of_element_located((By.XPATH, '//input[@id="changeButton"]')))
+        skift_button.click()
+
+        basic_email(["lfrmi@aarhus.dk", "du@msb.aarhus.dk"], f"Password ændret på {credential_name}",
+            f"""<html>
+            <body>
+            <p>Password på {credential_name} ændret fra {password} til {new_pw}</p>
+            <p></p>
+            <p>Husk at opdatere {credential_name} på OpenOrchestrator på Udvikling + Onenote</p>
+            </body>
+            </html>""")
+
+        return new_pw
 
 
 def modulus_login(username: str, password: str, drivertype: str = 'chrome'):
